#ifndef USE_METAL
attribute vec4 inPos;
attribute vec2 inUV0;
attribute vec2 inUV1;

uniform highp mat4 inWorldViewProjMatrix;
uniform highp mat4 inWorldMatrix;

varying highp   vec2  uv0;	
varying highp   vec2  uv1;

#if defined(USE_DECALS) || defined (USE_SNOWCOVERED)
	attribute vec2 inUV2;
#endif

#if defined(USE_DECALS)
	varying highp   vec2  uv2;
#endif

#if defined(USE_NDIS) || defined (USE_SNOWCOVERED) || defined(USE_VERTEX_NDIS)
	attribute vec4 inNormal;
#endif

#if defined(USE_VCOLOR_MASKB_TINT) || defined (USE_VCALPHA)
	attribute vec4 	inColors;
	varying lowp 	vec4  	VC;
#endif

#if defined(USE_SNOWCOVERED)
	uniform	mediump float snowUVMultip;
	varying highp   vec2  uvSnow0;
	varying highp   vec2  uvSnow1;
	varying lowp   	float  snowMask;
#endif

	uniform	mediump float 		inFogStart;
	uniform	mediump float 		inFogRangeI;
	uniform lowp 	vec4 		inFogColor;				
	varying lowp 	float 		fogRatio;

#if defined(USE_NDIS) || defined(USE_VERTEX_NDIS)
	varying highp 	vec2  VSN;
	uniform highp float reflectionBend;
	uniform mat4 inWorldViewITMatrix;
#endif

#if defined(USE_SHADOWMAP) && defined(USE_SHADOWS)
	uniform highp mat4 inLightTextureMatrix;
	varying highp vec2 lightProjectedUV;
	//varying highp float lightProjectedZ;	
#endif

#ifdef COMPUTE_LINEAR_DEPTH
	varying highp vec3 positionWorldSpace;
#endif

void main()
{
gl_Position = inWorldViewProjMatrix * vec4(inPos.xyz, 1.0);

uv0 = inUV0;
uv1 = inUV1;

#if defined(USE_SHADOWMAP) && defined(USE_SHADOWS)
	highp mat4 lm = inLightTextureMatrix * inWorldMatrix; //this is detected by the GPU and precomputed
	highp vec3 lightCoord  = (lm * vec4(inPos.xyz, 1.0)).xyz;
	lightProjectedUV = lightCoord.xy;
	//lightProjectedZ = min(lightCoord.z + 0.0003, 1.0);
#endif

#if defined(USE_SNOWCOVERED)
	uvSnow0 = inUV0 * snowUVMultip;
	uvSnow1 = inUV2 * snowUVMultip;
	snowMask = max((inWorldMatrix * vec4(inNormal.xyz, 0.0)).z, 0.0);
#endif

#if defined(USE_DECALS)
	uv2 = inUV2;
#endif

#if defined(USE_VCOLOR_MASKB_TINT) || defined (USE_VCALPHA)
	VC = inColors;
#endif

#if defined(USE_NDIS) || defined(USE_VERTEX_NDIS)
	highp vec2 fragPos 	= normalize(gl_Position.xyz).xy;
	#if defined(USE_NDISREFRACTION)
		highp vec3 VSN3d = (inWorldViewITMatrix *  vec4(-inNormal.xyz, 0.0)).xyz;
		VSN3d 	= normalize(VSN3d) * 0.5 + 0.5;
		VSN 	= VSN3d.xy - fragPos * reflectionBend;
	#else
		highp vec3 VSN3d = (inWorldViewITMatrix *  vec4(inNormal.xyz, 0.0)).xyz;
		VSN3d 	= normalize(VSN3d) * 0.5 + 0.5;
		VSN 	= VSN3d.xy + fragPos * reflectionBend;
	#endif
#endif
#if defined(USE_VERTEX_NDIS)
	VSN.y *= -1.0;
#endif

	lowp float fog = clamp((gl_Position.z - inFogStart) * inFogRangeI, 0.0, 1.0);
	fogRatio = fog * inFogColor.a;
	
#ifdef COMPUTE_LINEAR_DEPTH
	highp vec4 posWS = (inWorldMatrix * vec4(inPos.xyz, 1.0));
	positionWorldSpace = posWS.xyz;
#endif
}

#else

// metal code

void vs()
{
VertexOutput out;
float4 position = cb.inWorldViewProjMatrix * float4(in.inPos.xyz, 1.0);
out.position = position;

out.uv0 = in.inUV0;
out.uv1 = in.inUV1;

#if defined(USE_SHADOWMAP) && defined(USE_SHADOWS)
	float4x4 lm = cb.inLightTextureMatrix * cb.inWorldMatrix; //this is detected by the GPU and precomputed
	float3 lightCoord  = (lm * float4(in.inPos.xyz, 1.0)).xyz;
	out.lightProjectedUV = lightCoord.xy;
	//lightProjectedZ = min(lightCoord.z + 0.0003, 1.0);
#endif

#if defined(USE_SNOWCOVERED)
	out.uvSnow0 = in.inUV0 * cb.snowUVMultip;
	out.uvSnow1 = in.inUV2 * cb.snowUVMultip;
	out.snowMask = max((cb.inWorldMatrix * float4(in.inNormal.xyz, 0.0)).z, 0.0);
#endif

#if defined(USE_DECALS)
	out.uv2 = in.inUV2;
#endif

#if defined(USE_VCOLOR_MASKB_TINT) || defined (USE_VCALPHA)
	out.VC = in.inColors;
#endif

#if defined(USE_NDIS) || defined(USE_VERTEX_NDIS)
	float2 fragPos 	= normalize(position.xyz).xy;
	#if defined(USE_NDISREFRACTION)
		float3 VSN3d = (cb.inWorldViewITMatrix *  float4(-in.inNormal.xyz, 0.0)).xyz;
		VSN3d 	= normalize(VSN3d) * 0.5 + 0.5;
		out.VSN 	= VSN3d.xy - fragPos * cb.reflectionBend;
	#else
		float3 VSN3d = (cb.inWorldViewITMatrix *  float4(in.inNormal.xyz, 0.0)).xyz;
		VSN3d 	= normalize(VSN3d) * 0.5 + 0.5;
		out.VSN 	= VSN3d.xy + fragPos * cb.reflectionBend;
	#endif
#endif
#if defined(USE_VERTEX_NDIS)
	out.VSN.y *= -1.0;
#endif

	half fog = clamp((position.z - cb.inFogStart) * cb.inFogRangeI, 0.0, 1.0);
	out.fogRatio = fog * (half)cb.inFogColor.a;
	
#ifdef COMPUTE_LINEAR_DEPTH
	float4 posWS = (cb.inWorldMatrix * float4(in.inPos.xyz, 1.0));
	out.positionWorldSpace = posWS.xyz;
#endif
return out;
}

#endif
